@@ -18,10 +18,8 @@ STOPWORDS = ['i', 'me', 'my', 'myself', 'we', 'our', 'ours', 'ourselves', 'you',
 'some', 'such', 'only', 'own', 'same', 'so', 'than', 'too', 'very','can', 'just',
 'should', "should(\'?)ve", 'now']
 
-punc = "!|\"|\#|\$|\%|\&|\'|\(|\)|\*|\+|\,|\-|\.|\/|\:|\;|\<|\=|\>|\?|\@|\[|\\|\]|\^|\_|\`|\{|\||\}|\~"
-
 TOPICWORDS = [{"token": "no", "tokenType": "negation"},{"token": "negative", "tokenType": "negation"},
-{"token": "not", "tokenType": "negation"},{"token": "nor", "tokenType": "negation"},
+{"token": "not", "tokenType": "negation"},{"token": "nor", "tokenType": "negation"}, {"token": "non-*", "suffix": "[A-Za-z]+", "tokenType": "negation"}, "([A-Za-z]+-)+\w+",
 {"token": "ain(\'?)t", "tokenType": "negation"},{"token": "aren(\'?)t", "tokenType": "negation"},
 {"token": "couldn(\'?)t", "tokenType": "negation"},{"token": "didn(\'?)t", "tokenType": "negation"},
 {"token": "doesn(\'?)t", "tokenType": "negation"},{"token": "hadn(\'?)t", "tokenType": "negation"},
@@ -33,7 +31,7 @@ TOPICWORDS = [{"token": "no", "tokenType": "negation"},{"token": "negative", "to
 {"token": "won(\'?)t", "tokenType": "negation"},{"token": "don(\'?)t", "tokenType": "negation"},
 {"token": "wouldn(\'?)t", "tokenType": "negation"},{"token": "denies", "tokenType": "negation"},
 {"token": "denied", "tokenType": "negation"},{"token": "\d+\.\d+", "tokenType": "numeric", "suffix": "[^\s\d-]+"},
-{"token": "\d+", "tokenType": "numeric","suffix": "[^\s\d-]+"},"([A-Za-z]("+punc+")?){1,2}[A-Za-z]",
+{"token": "\d+", "tokenType": "numeric","suffix": "[^\s\d-]+"},"([A-Za-z](\.)?){1,2}[A-Za-z]",
 {"token": "\d+\.\d+", "tokenType": "numeric"},{"token": "\d+", "tokenType": "numeric"},
 {"token": "one", "tokenType": "numeric"},{"token": "two", "tokenType": "numeric"},
 {"token": "three", "tokenType": "numeric"},{"token": "four", "tokenType": "numeric"},
@@ -108,19 +106,19 @@ def pullOutTopics(topics, text, leftovers, noise):
         for match in re.finditer("(\s|"+"|\\".join(punc)+"|^)"+token+suffix+"(\s|"+"|\\".join(punc)+"\s|$)", text.lower(),overlapped=True):
             topicmatches = re.finditer(token, text[match.start():match.end()].lower())
             allindexes = allindexes.union(range(match.start(), match.end()))
+
             for top in topicmatches:
                 potentialphrases.append((match.start()+ top.start(),match.start()+ top.end(), tokenType))
                 if suffix == "":
                     continue
                 suffixes = re.finditer(suffix, text[match.start()+top.end():match.end()].lower())
+
                 for suff in suffixes:
                     if suff.start() == suff.end():
                         continue
                     potentialphrases.append((match.start()+ top.end()+suff.start(),match.start()+ top.end()+suff.end(), "token"))
-
     for potential in potentialphrases:
         leftovers, ret = checkAndRemoveLeftovers(leftovers, potential[0], potential[1], ret, potential[2])
-
     ## what's left is noise
     allindexes = leftovers.intersection(allindexes)
     noise = pullOutTokens(allindexes, "noise") + noise
@@ -209,15 +207,54 @@ def wildgram(text, stopwords=STOPWORDS, topicwords=TOPICWORDS, include1gram=True
 class WildRules:
     def __init__(self, rules):
         self.rules = rules
+        self.spans = {}
+        for i in range(len(rules)):
+            rule = rules[i]
+            if rule["spanType"] not in self.spans:
+                self.spans[rule["spanType"]] = {}
+            for span in rule["spans"]:
+                if span not in self.spans[rule["spanType"]]:
+                    self.spans[rule["spanType"]][span] =[]
+                self.spans[rule["spanType"]][span].append(i)
+
+
+
     def apply(self, span):
         tokens = wildgram(span, includeParent = False)
         if len(tokens) == 0:
             return []
+        applied = {}
         for i in range(len(tokens)):
             tokens[i]["topic"] = "unknown"
-            for rule in self.rules:
-                if replaceMeaninglessPunc(tokens[i][rule["spanType"]]) in rule["spans"]:
-                    tokens[i]["topic"] = rule["topic"]
+            for spanType in self.spans:
+                if replaceMeaninglessPunc(tokens[i][spanType]) in self.spans[spanType]:
+                    for ruleApplied in self.spans[spanType][replaceMeaninglessPunc(tokens[i][spanType])]:
+                        if ruleApplied not in applied:
+                            applied[ruleApplied] = []
+                        applied[ruleApplied].append(i)
+
+        for i in sorted(list(applied.keys())):
+            rule = self.rules[i]
+            for j in applied[i]:
+                if "nearby" not in rule:
+                    tokens[j]["topic"] =  rule["topic"]
+                    continue
+                for nearby in rule["nearby"]:
+                    start = j - 10
+                    end = j + 10
+                    if "startOffset" in nearby:
+                        start = j + nearby["startOffset"]
+                    if "endOffset" in nearby:
+                        end = j + nearby["endOffset"]
+                    for k in range(start, end):
+                        if k < 0:
+                            continue
+                        if k >= len(tokens):
+                            continue
+                        if replaceMeaninglessPunc(tokens[k][nearby["spanType"]]) in nearby["spans"]:
+                            tokens[j]["topic"] = rule["topic"]
+
+
         newtoks = []
         prevType = tokens[0]["topic"]
         prevToks = [tokens[0]]
